@@ -54,6 +54,7 @@ struct
 	get_d: 'a option;
 	set: 'a -> unit;
 	get: 'a;
+        on_change: ('a -> unit) -> unit
       >
 
   type links = (string * ut) list
@@ -105,6 +106,8 @@ struct
     val value_d : 'a option ref = ref d
     val value : 'a option ref = ref None
 
+    val mutable listeners : ('a -> unit) list = []
+
     initializer
       p self#ut;
       List.iter (fun (s, t) -> self#plug s t) l
@@ -185,7 +188,12 @@ struct
       | Some v -> v
       end
 
-    method set (v : 'a) : unit = value := Some v
+    method set (v : 'a) : unit =
+      value := Some v;
+      List.iter (fun fn -> fn v) listeners
+
+    method on_change (fn : 'a -> unit) : unit =
+      listeners <- fn::listeners
 
   end
 
@@ -218,16 +226,6 @@ struct
   let string_of_path p =
     String.concat "." p
 
-  let routes (t : ut) (st : ut) =
-    let rec aux l t =
-      begin match t = st with
-      | true -> [List.rev l]
-      | false ->
-	  List.concat (List.map (fun s -> aux (s :: l) (t#path [s])) t#subs)
-      end
-    in
-    aux [] t
-
   let get_string (t : ut) =
     begin try
        	begin match t#kind with
@@ -371,7 +369,7 @@ struct
 	  then ()
 	  else
 	    begin try conf_set t l with
-	    | Wrong_Conf (x,y) ->
+	    | Wrong_Conf (_,y) ->
 		raise (File_Wrong_Conf (s,!nb,y))
 	    end
 	done
@@ -479,14 +477,14 @@ struct
     stop.depends <- a :: stop.depends;
     a
 
-  let rec exec a =
+  let exec a =
     let log =
       if conf_trace#get then
 	     begin fun s ->
 	      let id = Thread.id (Thread.self ()) in
 	      Printf.printf "init(%i):%-35s@%s\n%!" id a.name s
 	     end
-      else begin fun s -> () end
+      else begin fun _ -> () end
     in
     let rec exec a =
       log "called";
@@ -543,6 +541,9 @@ struct
   "ocaml-dtools not compiled with ocaml >= 3.11, \
    cannot print stack backtrace"
 
+  (* For the compiler.. *)
+  let () = ignore (get_backtrace ())
+
   open Printexc
 
   let main f () =
@@ -665,7 +666,7 @@ struct
     if prohibit_root then exit_when_root ();
     if conf_daemon#get && Sys.os_type <> "Win32" then
       daemonize () ;
-    let signal_h i = () in
+    let signal_h _ = () in
     Sys.set_signal Sys.sigterm (Sys.Signal_handle signal_h);
     Sys.set_signal Sys.sigint (Sys.Signal_handle signal_h);
     (* We block signals that would kill us,
@@ -849,34 +850,36 @@ struct
 	    let time = Unix.gettimeofday () in
 	    Printf.ksprintf (fun s -> proceed (time, label lvl ^ " " ^ s))
 	| false ->
-	    Printf.ksprintf (fun s -> ())
+	    Printf.ksprintf (fun _ -> ())
 	end
     end
 
   let init () =
     let time = Unix.gettimeofday () in
-    if conf_file#get then
-      begin
-	let opts =
-	  [Open_wronly; Open_creat; Open_nonblock]
-	  @ (if conf_file_append#get then [Open_append] else [Open_trunc])
-	in
-	let log_file_path = conf_file_path#get in
-	let log_file_perms = conf_file_perms#get in
-	(* Re-open log file on SIGUSR1 -- for logrotate *)
-        if Sys.os_type <> "Win32" then
-	 Sys.set_signal Sys.sigusr1
-	  (Sys.Signal_handle
-              begin fun _ ->
-		begin match !log_ch with
-		| None -> ()
-		| Some ch -> log_ch := None; close_out ch;
-		end;
-		log_ch := Some (open_out_gen opts log_file_perms log_file_path)
-	      end
-	  );
-	log_ch := Some (open_out_gen opts log_file_perms log_file_path);
-      end;
+    let reopen_log =
+      if conf_file#get then
+        begin
+          let opts =
+	    [Open_wronly; Open_creat; Open_nonblock]
+	    @ (if conf_file_append#get then [Open_append] else [Open_trunc])
+	  in
+	  let log_file_path = conf_file_path#get in
+	  let log_file_perms = conf_file_perms#get in
+	  log_ch := Some (open_out_gen opts log_file_perms log_file_path);
+          (fun _ ->
+            begin
+              match !log_ch with
+                | None -> ()
+                | Some ch -> log_ch := None; close_out ch;
+            end;
+            log_ch := Some (open_out_gen opts log_file_perms log_file_path))
+        end
+      else (fun _ -> ())
+    in
+    (* Re-open log file on SIGUSR1 -- for logrotate *)
+    if Sys.os_type <> "Win32" then
+     Sys.set_signal Sys.sigusr1
+      (Sys.Signal_handle reopen_log);
     mutexify (fun () ->
       print (time, ">>> LOG START");
       begin match !state with
